import re
import json
import random
import string
import pdb


def get_solver_prompt(prompt_type, params=None):
	sys_prompt = ""
	prompt = ""

	if prompt_type == "8-shot-cot":
		sys_prompt = "You are an expert mathematician. Your final statement must be of the form 'The answer is <answer>'."
		prompt = f"""Solve the final math word problem given below by thinking step-by-step. You should always work with exact numbers - never round down or round up decimals based on context. Give the final answer in the end by saying "The answer is <number>".

Q: There are 15 trees in the grove. Grove workers will plant trees in the grove today. After they are done, there will be 21 trees. How many trees did the grove workers plant today?
A: There are 15 trees originally. Then there were 21 trees after some more were planted. So there must have been 21 - 15 = 6. The answer is 6.
Q: If there are 3 cars in the parking lot and 2 more cars arrive, how many cars are in the parking lot?
A: There are originally 3 cars. 2 more cars arrive. 3 + 2 = 5. The answer is 5.
Q: Leah had 32 chocolates and her sister had 42. If they ate 35, how many pieces do they have left in total?
A: Originally, Leah had 32 chocolates. Her sister had 42. So in total they had 32 + 42 = 74. After eating 35, they had 74 - 35 = 39. The answer is 39.
Q: Jason had 20 lollipops. He gave Denny some lollipops. Now Jason has 12 lollipops. How many lollipops did Jason give to Denny?
A: Jason started with 20 lollipops. Then he had 12 after giving some to Denny. So he gave Denny 20 - 12 = 8. The answer is 8.
Q: Shawn has five toys. For Christmas, he got two toys each from his mom and dad. How many toys does he have now?
A: Shawn started with 5 toys. If he got 2 toys each from his mom and dad, then that is 4 more toys. 5 + 4 = 9. The answer is 9.
Q: There were nine computers in the server room. Five more computers were installed each day, from monday to thursday. How many computers are now in the server room?
A: There were originally 9 computers. For each of 4 days, 5 more computers were added. So 5 * 4 = 20 computers were added. 9 + 20 is 29. The answer is 29.
Q: Michael had 58 golf balls. On tuesday, he lost 23 golf balls. On wednesday, he lost 2 more. How many golf balls did he have at the end of wednesday?
A: Michael started with 58 golf balls. After losing 23 on tuesday, he had 58 - 23 = 35. After losing 2 more, he had 35 - 2 = 33 golf balls. The answer is 33.
Q: Olivia has $23. She bought five bagels for $3 each. How much money does she have left?
A: Olivia had 23 dollars. 5 bagels for 3 dollars each will be 5 x 3 = 15 dollars. So she has 23 - 15 dollars left. 23 - 15 is 8. The answer is 8.
Q: {params}
A:"""
	
	return prompt, sys_prompt


def get_verification_prompt(prompt_type, params=None):
	sys_prompt = ""
	prompt = ""

	if prompt_type == "check_banned_info":
		sys_prompt = "You are an expert at verifying data. Your final statement must start with 'yes' or 'no'."
		prompt = f"""You are given a math word problem and some adversarial information. You must check whether any of the facts provided in the adversarial information is mentioned or contradicted in the problem. 

Do this step by step for each fact in the adversarial information. If you find a fact that is (1) mentioned or (2) contradicted, you must say 'yes' and explain how. If you found any fact that was mentioned or contradicted, then in your final statement, you must say "yes, facts were mentioned or contradicted". Otherwise, if none of the facts have been mentioned or contradicted, say "no, facts were not mentioned or contradicted" in the final statement.

Question: {params[0]}
Adversarial Information: {params[1]}"""

	elif prompt_type == "grammar_correct":
		sys_prompt = "You are an expert English and Mathematics teacher."
		prompt = f"""You are given three pieces of a math word problem's context. Your task is to correct the grammar, punctuation and language of all the three pieces. Each context should consist of properly ended sentences, and should not pose any questions. Make sure you do not change any important information or fact, and that you do not add any new information. Also make sure that the three pieces of context flow easily and logically from one to the next.

Context 1: {params[0]}

Context 2: {params[1]}

Context 3: {params[2]}

Give output in the following format only:
Corrected context 1: <>

Corrected context 2: <>

Corrected context 3: <>"""
	
	return prompt, sys_prompt


def get_generator_prompt(prompt_type, params=None):
	sys_prompt = ""
	prompt = ""

	if prompt_type == "problem_extend":
		sys_prompt = "You are an expert mathematician."
		if params is None:
			return "", sys_prompt
		prompt = f"""You are a research scientist. Your task is to create a hard math word problem to test an advanced math reasoning system. For that, you are given the following problem:

Q: {params[0]}
A: {params[1]}

Your job is to first divide up the problem into the "context" and the "question statement". Isolate the quantity that the problem is inquiring about by looking at the final question statement and the rest of the information provided becomes the context. Also form a brief answer statement by phrasing the answer in a complete sentence. Do not include the answer statement in the context.

Then you need to further continue the problem over that quantity, by introducing a scenario and final question where you need to perform one more operation (such as +,-,/,*, etc.) over this quantity to get the final answer. Crucially, the new context must not mention the original answer - it still has to be inferred based on previous information. Do not make any calculation or inference in the new context. Try to make the new context challenging. Also provide a complete reasoning of how you reached the new answer (never round down or round up decimals).

Give output in the following format only:
Original context [without question statement]: <>
Question statement: <>
Original answer: <>
Original answer statement: <>

New operation over original answer: <>
New context [Do not mention original answer]: <>
New question statement: <>
New answer reasoning: <>
New answer [Number only]: <>"""
	
	elif prompt_type == "problem_extend_small_model":
		sys_prompt = "You are an expert mathematician. Do not output <>. Do not make assumptions in your reasoning. Do not output any seed question or seed answer. Limit your new answer reasoning to 2-3 sentences. End your output with the new answer in numerical form (without calculation or equation)."
		if params is None:
			return "", sys_prompt
		prompt = f"""You are a research scientist. Your task is to create a hard math word problem to test an advanced math reasoning system. For that, you are given the following problem:

Q: {params[0]}
A: {params[1]}

Your job is to first divide up the problem into the "context" and the "question statement". Isolate the quantity that the problem is inquiring about by looking at the final question statement and the rest of the information provided becomes the context. Also form a brief answer statement by phrasing the answer in a complete sentence. Do not include the answer statement in the context.

Then you need to further continue the problem over that quantity, by introducing a scenario and final question where you need to perform one more operation (such as +,-,/,*, etc.) over this quantity to get the final answer. Do not use any other information from the original question, you just have to assume the information in the original answer statement and build on that. Crucially, the new context must not mention the original answer - it still has to be inferred based on previous information. Do not make any calculation or inference in the new context. Also provide a complete reasoning of how you reached the new answer (never round down or round up decimals).

Give output in the following format only:
Original context: <>
Question statement: <>
Original answer: <>
Original answer statement: <>

New operation over original answer: <>
New context [Do not mention original answer]: <>
New question statement: <>
New answer reasoning: <>
New answer: <>"""
	
	elif prompt_type == "problem_bottom_up":
		sys_prompt = "You are an expert mathematician."
		prompt = f"""You are a research scientist. Your task is to create a hard math word problem to test an advanced math reasoning system. For that, you are given a problem.

Your job is to first divide up the problem into the "context" and the "question statement". Isolate the quantity that the problem is inquiring about by looking at the final question statement and the rest of the information provided becomes the context.

Now choose one of the quantities specified in the information provided in the context of the question, and modify that context to not include that information (do not add anything new, especially do not add the answer).

Next, you must create a new independent question whose answer is going to be that quantity, by assuming new information or background. The new question should ask about exactly the same objects associated with the chosen quantity.

Given below are a few examples:

Example 1:
Original question: Jack had 8 pens. Jack gave 3 pens to Mary. How many pens does Jack have now?
Original answer: Jack had 8 pens. Since Jack gave 3 pens to Mary, he now has 8 - 3 = <<8-3=5>>5 pens.
#### 5

Original context [without question statement]: Jack had 8 pens. Jack gave 3 pens to Mary.
Question statement: How many pens does Jack have now?

Number chosen [present in original question, is to be asked by new question]: 3
Modified original context [without including the chosen number & without question statement]: Jack had 8 pens.

New question statement based on chosen quantity: How many pens did Jack give to Mary?
Complete new question: Max gave 6 pens to Mary while Jack gave half as many to her. How many pens did Jack give to Mary?
Context information in new question [without any question statement]: Max gave 6 pens to Mary while Jack gave half as many to her.
New reasoning: Jack gave half as many pens as Max, so Jack gave 6 / 2 = <<6/2=3>>3 pens to Mary.

Example 2:
Original question: Sam made 8 dollars mowing lawns over the Summer. He charged 2 bucks for each lawn. How many lawns did he mow?
Original answer: Sam mowed 8 / 2 = <<8/2=4>>4 lawns.
#### 4

Original context [without question statement]: Sam made 8 dollars mowing lawns over the Summer. He charged 2 bucks for each lawn.
Question statement: How many lawns did he mow?

Number chosen [present in original question, is to be asked by new question]: 2
Modified original context [without including the chosen number & without question statement]: Sam made 8 dollars mowing lawns over the Summer.

New question statement based on chosen quantity: How much did Sam charge to mow a single lawn?
Complete new question: The running rate of mowing lawns is 5 bucks per lawn. Sam was desperate for work, so he only charged 40% of the running rate. How much did Sam charge to mow a single lawn?
Context information in new question [without any question statement]: The running rate of mowing lawns is 5 bucks per lawn. Sam was desperate for work, so he only charged 40% of the running rate.
New reasoning: Sam charged 0.4 * 5 = $<<0.4*5=2>>2 per lawn.

Example 3:
Original question: Lucy made some juice from fresh oranges. If she used 2 oranges per glass of juice and she made 6 glasses of juice, how many oranges did she use?
Original answer: Lucy used 2 * 6 = <<2*6=12>>12 oranges.
#### 12

Original context [without question statement]: Lucy made some juice from fresh oranges. She used 2 oranges per glass of juice and she made 6 glasses of juice.
Question statement: How many oranges did she use?

Number chosen [present in original question, is to be asked by new question]: 6
Modified original context [without including the chosen number & without question statement]: Lucy made some juice from fresh oranges. She used 2 oranges per glass of juice.

New question statement based on chosen quantity: How many glasses of juice did Lucy make?
Complete new question: Lucy's four friends came over during the summer to drink orange juice. Two of her friends wanted 2 glasses each while the others wanted 1 glass each. How many glasses of juice did Lucy make?
Context information in new question [without any question statement]: Lucy's four friends came over during the summer to drink orange juice. Two of her friends wanted 2 glasses each while the others wanted 1 glass each.
New reasoning: Two friends wanted 2 glasses each, so that is 2 * 2 = 4 glasses. The other two friends wanted 1 glass each, so that is 2 * 1 = 2 glasses. In total, Lucy made 4 + 2 = <<4+2=6>>6 glasses of juice.

Now, do the same for the following original problem:

Q: {params[0]}
A: {params[1]}

You must follow these rules:
1. The new question must involve at least one mathematical operation whose answer is the chosen number.
2. The new question must not mention any information or answer from the original question.
3. The new question must not mention the chosen quantity value directly.

Give output in the following format only:
Original context [without question statement]: <>
Question statement: <>

Number chosen [present in original question, is to be asked by new question]: <>
Modified original context [without including the chosen number & without question statement]: <>

New question statement based on chosen quantity: <>
Complete new question: <>
Context information in new question [without any question statement]: <>
New reasoning: <>"""
	
	elif prompt_type == "problem_bottom_up_break":
		sys_prompt = "You are an expert mathematician."
		prompt = f"""You are a research scientist. Your task is to break down math word problems to test an advanced math reasoning system. For that, you are given a problem.

Your job is to first divide up the problem into the "context" and the "question statement". Isolate the quantity that the problem is inquiring about by looking at the final question statement and the rest of the information provided becomes the context. Also form a brief answer statement by phrasing the answer in a complete sentence.

Now segregate all of the quantities specified in the information provided in the context of the question, and modify the corresponding contexts to not include the information about that quantity (do not add anything new, especially do not add the answer). Also pose the question statement asking about that quantity. This question statement should be comprehensive - it should mention the objects and subjects associated with the chosen quantity. Also separately mention the subjects and objects involved in the new question statement.

Given below are a few examples:

Example 1:
Original question: Jack had 8 pens. Jack gave 3 pens to Mary. How many pens does Jack have now?
Original answer: Jack had 8 pens. Since Jack gave 3 pens to Mary, he now has 8 - 3 = <<8-3=5>>5 pens.
#### 5

Original context [without question statement]: Jack had 8 pens. Jack gave 3 pens to Mary.
Question statement: How many pens does Jack have now?
Answer statement: Jack has 5 pens now.

Quantity 1:
Number chosen [present in original question, is to be asked by new question]: 3
Modified original context [without including the chosen number & without question statement]: Jack had 8 pens.
New question statement based on chosen quantity [comprehensive, mentioning subjects and objects]: How many pens did Jack give to Mary?
Subjects involved in new question statement: Jack, Mary
Objects involved in new question statement: pens

Quantity 2:
Number chosen [present in original question, is to be asked by new question]: 8
Modified original context [without including the chosen number & without question statement]: Jack gave 3 pens to Mary.
New question statement based on chosen quantity [comprehensive, mentioning subjects and objects]: How many pens did Jack have?
Subjects involved in new question statement: Jack
Objects involved in new question statement: pens

Example 2:
Original question: Sam made 8 dollars mowing lawns over the Summer. He charged 2 bucks for each lawn. How many lawns did he mow?
Original answer: Sam mowed 8 / 2 = <<8/2=4>>4 lawns.
#### 4

Original context [without question statement]: Sam made 8 dollars mowing lawns over the Summer. He charged 2 bucks for each lawn.
Question statement: How many lawns did he mow?
Answer statement: Sam mowed 4 lawns.

Quantity 1:
Number chosen [present in original question, is to be asked by new question]: 2
Modified original context [without including the chosen number & without question statement]: Sam made 8 dollars mowing lawns over the Summer.
New question statement based on chosen quantity [comprehensive, mentioning subjects and objects]: How much did Sam charge to mow a single lawn?
Subjects involved in new question statement: Sam
Objects involved in new question statement: dollars, lawn

Quantity 2:
Number chosen [present in original question, is to be asked by new question]: 8
Modified original context [without including the chosen number & without question statement]: Sam charged 2 bucks for mowing each lawn.
New question statement based on chosen quantity [comprehensive, mentioning subjects and objects]: How much money did Sam make mowing lawns?
Subjects involved in new question statement: Sam
Objects involved in new question statement: dollars, lawn

Example 3:
Original question: Lucy made some juice from fresh oranges. If she used 2 oranges per glass of juice and she made 6 glasses of juice, how many oranges did she use?
Original answer: Lucy used 2 * 6 = <<2*6=12>>12 oranges.
#### 12

Original context [without question statement]: Lucy made some juice from fresh oranges. She used 2 oranges per glass of juice and she made 6 glasses of juice.
Question statement: How many oranges did she use?
Answer statement: Lucy used 12 oranges to make juice.

Quantity 1:
Number chosen [present in original question, is to be asked by new question]: 6
Modified original context [without including the chosen number & without question statement]: Lucy made some juice from fresh oranges. She used 2 oranges per glass of juice.
New question statement based on chosen quantity [comprehensive, mentioning subjects and objects]: How many glasses of orange juice did Lucy make?
Subjects involved in new question statement: Lucy
Objects involved in new question statement: glasses, orange juice

Quantity 2:
Number chosen [present in original question, is to be asked by new question]: 2
Modified original context [without including the chosen number & without question statement]: Lucy made some juice from fresh oranges. She made 6 glasses of juice.
New question statement based on chosen quantity [comprehensive, mentioning subjects and objects]: How many oranges did Lucy use per glass of juice?
Subjects involved in new question statement: Lucy
Objects involved in new question statement: glasses, orange juice

Now, do the same for the following original problem:

Q: {params[0]}
A: {params[1]}

Give output in the following format only:
Original context [without question statement]: <>
Question statement: <>
Answer statement: <>

Quantity <>:
Number chosen [present in original question, is to be asked by new question]: <>
Modified original context [without including the chosen number & without question statement]: <>
New question statement based on chosen quantity [comprehensive, mentioning subjects and objects]: <>
Subjects involved in new question statement: <>
Objects involved in new question statement: <>

Quantity <>:
..."""
	
# 	elif prompt_type == "problem_bottom_up_create_1":
# 		sys_prompt = "You are an expert mathematician."
# 		prompt = f"""You are a research scientist. Your task is to create a challenging math word problem to test an advanced math reasoning system. 

# You are given a question statement and corresponding answer. Your job is to come up with a context of information, which when combined with the question statement, will lead to the answer provided. Make sure the context has sensible and plausible information. The final question statement and answer of the new problem you create must be exactly as provided.

# Important instructions you must follow:
# 1. Ensure that the new problem you create (1) does not mention and (2) does not contradict the restricted information provided with the question statement and answer.
# 2. One good way to create new problems is to use completely new characters or objects and create a scenario different from the one mentioned in the restricted information.
# 3. Use very different numbers (but the answer still has to be the one given).

# Given below are a few examples:

# Example 1:

# Question statement: How many pens did Jack give to Mary?
# Answer: 3

# Restricted information not to be repeated or contradicted:
# - Jack had 8 pens.
# - Jack has 5 pens now.

# Final question statement of the new problem should ask about these subjects and objects:
# Subjects: Jack, Mary
# Objects: pens
# Crafting problem with final question statement as "How many pens did Jack give to Mary?"...

# Complete new problem: Max gave a dozen pens to Mary while Jack gave 25% of that to her. How many pens did Jack give to Mary?
# Context information in new problem [without any question statement]: Max gave a dozen pens to Mary while Jack gave 25% of that to her.
# New reasoning: Jack gave 0.25 * 12 = <<0.25*12>>3 pens to Mary.

# Example 2:

# Question statement: How much did Sam charge to mow a single lawn?
# Answer: 2

# Restricted information not to be repeated or contradicted:
# - Sam made 8 dollars mowing lawns over the Summer.
# - Sam mowed 4 lawns in total.

# Final question statement of the new problem should ask about these subjects and objects:
# Subjects: Sam
# Objects: dollars, lawn
# Crafting problem with final question statement as "How much did Sam charge to mow a single lawn?"...

# Complete new problem: The running rate of mowing lawns is 6 bucks per lawn. Sam was desperate for work, so he mowed 3 lawns for that much money. How much did Sam charge to mow a single lawn?
# Context information in new problem [without any question statement]: The running rate of mowing lawns is 6 bucks per lawn. Sam was desperate for work, so he mowed 3 lawns for that much money.
# New reasoning: Sam charged 6 / 3 = <<6/3>>2 bucks per lawn.

# Example 3:

# Question statement: How many glasses of orange juice did Lucy make?
# Answer: 6

# Restricted information not to be repeated or contradicted:
# - Lucy used 2 oranges per glass of juice.
# - Lucy used 12 oranges to make juice.

# Final question statement of the new problem should ask about these subjects and objects:
# Subjects: Lucy
# Objects: glasses, orange juice
# Crafting problem with final question statement as "How many glasses of orange juice did Lucy make?"...

# Complete new problem: Lucy's four friends came over during the summer to drink orange juice. Two of her friends wanted 2 glasses each while the others wanted 1 glass each. How many glasses of orange juice did Lucy make?
# Context information in new problem [without any question statement]: Lucy's four friends came over during the summer to drink orange juice. Two of her friends wanted 2 glasses each while the others wanted 1 glass each.
# New reasoning: Two friends wanted 2 glasses each, so that is 2 * 2 = 4 glasses. The other two friends wanted 1 glass each, so that is 2 * 1 = 2 glasses. In total, Lucy made 4 + 2 = <<4+2=6>>6 glasses of juice.

# went to see the oranges being harvested. He found out that they harvest 83 sacks
# per day and that each sack contains 12 oranges. 

# Example 4:

# Question statement: How many sacks of cherries do they harvest per day?
# Answer: 72

# Restricted information not to be repeated or contradicted:
# - Each sack of cherries contains 3 cherries.
# - They harvest 216 cherries per day.

# Final question statement of the new problem should ask about these subjects and objects:
# Subjects: Harvesters
# Objects: sacks of cherries
# Crafting problem with final question statement as "How many sacks of cherries do they harvest per day?"...

# Complete new problem: The harvesters harvest 4 times as many sacks of cherries as they do blueberries. If they harvest 18 sacks of blueberries per day, how many sacks of cherries do they harvest per day?
# Context information in new problem [without any question statement]: The harvesters harvest 4 times as many sacks of cherries as they do blueberries. They harvest 18 sacks of blueberries per day.
# New reasoning: The harvesters harvest 4 * 18 = <<4*18=72>>72 sacks of cherries per day.

# Example 5:

# Question statement: How many boys are in the school?
# Answer: 250

# Restricted information not to be repeated or contradicted:
# - There are 400 girls in the school.
# - There are total 650 students in the school.

# Final question statement of the new problem should ask about these subjects and objects:
# Subjects: boys
# Objects: school
# Crafting problem with final question statement as "How many boys are in the school?"...

# Complete new problem: There are 370 students in the university. If the number of boys in the school is 120 less than the number of students in the University, how many boys are in the school?
# Context information in new problem [without any question statement]: There are 370 students in the university. The number of boys in the school is 120 less than the number of students in the University.
# New reasoning: The number of boys in school is 370 - 120 = <<370-120=250>>250.

# Example 6:

# Question statement: What percentage off was the handbag Kelly bought?
# Answer: 30

# Restricted information not to be repeated or contradicted:
# - Original price of the handbag was $60.
# - Kelly paid $42 for the handbag.

# Final question statement of the new problem should ask about these subjects and objects:
# Subjects: Kelly
# Objects: handbag
# Crafting problem with final question statement as "What percentage off was the handbag Kelly bought?"...

# Complete new problem: The sale at the shopping mall on luxury items began today. The discount percentage on handbags is half as much as the discount on shoes, while the discount percentage on shoes is three times that as the discount on dresses. If the discount on dresses is 20%, what percentage off was the handbag Kelly bought?
# Context information in new problem [without any question statement]: The sale at the shopping mall on luxury items began today. The discount percentage on handbags is half as much as the discount on shoes, while the discount percentage on shoes is three times that as the discount on dresses. The discount on dresses is 20%.
# New reasoning: The discount on shoes is 3 * 20 = 60%. The discount on handbags is half of that, so it is 60 / 2 = <<60/2=30>>30%.

# Do the same for the following problem:

# Question statement: {params[0]}
# Answer: {params[1]}

# Restricted information not to be repeated or contradicted:
# {params[2]}

# Final question statement of the new problem should ask about these subjects and objects:
# Subjects: {params[3]}
# Objects: {params[4]}
# Crafting problem with final question statement as "{params[0]}"...

# Give output in the following format only:
# Complete new problem: <>
# Context information in new problem [without any question statement]: <>
# New reasoning: <>"""

	elif prompt_type == "problem_bottom_up_create_1":
		sys_prompt = "You are an expert mathematician. Do not output any # symbols."
		prompt = f"""You are a research scientist. Your task is to create a challenging math word problem to test an advanced math reasoning system. 

You are given a question statement and corresponding answer. Your job is to come up with a creative context of information, which when combined with the question statement, will lead to the answer provided. Make sure the context has sensible and plausible information. The final question statement and answer of the new problem you create must be exactly as provided.

Important instructions you must follow:
1. Ensure that the new problem you create (1) does not mention and (2) does not contradict the restricted information provided with the question statement and answer.
2. Use very different numbers (but the answer still has to be the one given).

Procedure to follow:
1. First think of new characters and objects to introduce in the context.
2. Decide on a mathematical operation such as addition, subtraction, multiplication, or division.
3. Then create the problem with the new context using the chosen operation which will lead to the answer provided. The final question statement must be exactly as provided below.

Question statement: {params[0]}
Answer: {params[1]}

Restricted information not to be repeated or contradicted:
{params[2]}

Final question statement of the new problem should ask about these subjects and objects:
Subjects: {params[3]}
Objects: {params[4]}
Crafting problem with final question statement as "{params[0]}"...

Follow the procedure I outlined above step by step. 

Towards the end, give output in exactly the following format:
Complete new problem (context + question statement): <>
Context information in new problem [without any question statement]: <>
New reasoning: <>"""
	
	elif prompt_type == "naive_baseline":
		sys_prompt = "You are an expert mathematician. If possible, avoid creating problems requiring calculations with decimal points. You must provide final numerical answer in the form of #### <number> in the end."
		prompt = f"""I want you to act as a problem rewriter.
Your objective is to rewrite a given math word problem into a more complex version to make those famous AI systems (e.g., ChatGPT and GPT4) a bit harder to handle. But the rewritten problem must be correct, reasonable and must be understood and responded by humans. You SHOULD complicate the given problem using the following method:
If given problem can be solved with just a few simple thinking processes, you can rewrite it to explicitly request multiple-step reasoning.

Given problem: {params[0]}
Given answer: {params[1]}

Give output in the following format only:
Rewritten problem: <>
New answer: <>
"""

	return prompt, sys_prompt